local function ServerSave(name)
    local seconds = game.tick / 60
    local hours = string.format("%04.f", math.floor(seconds / 3600))
    local mins = string.format("%02.f", math.floor(seconds / 60 - (hours * 60)))
    local secs = string.format("%02.f", math.floor(seconds - hours * 3600 - mins * 60))
    local time = string.format("%s_%s_%s", hours, mins, secs)

    local seed = game.surfaces[1].map_gen_settings.seed
    local save_name = string.format("%s_%s_%s", seed, time, name)
    if settings.global["MileStoneSaves_name"].value ~= "" then
        save_name = string.format("%s_%s_%s", settings.global["MileStoneSaves_name"].value, time, name)
    end

    if game.is_multiplayer() then
        game.server_save(save_name)
    else
        game.auto_save(save_name)
    end
end

local function IntervalServerSave(event)
    if event.tick == 0 then
        return
    end
    ServerSave("periodic")
end

local function ProductionChecks(event)
    if event.tick == 0 then
        return
    end
    if settings.global["MileStoneSaves_science"].value then
        local produced = {}
        for _, force in pairs(game.forces) do
            for _, surface in pairs(game.surfaces) do
                for item, count in pairs(force.get_item_production_statistics(surface.name).input_counts) do
                    if not produced[item] then
                        produced[item] = 0
                    end
                    produced[item] = produced[item] + count
                end
            end
            local items = prototypes.get_item_filtered({ { filter = "type", type = "tool" } })
            for _, item in pairs(items) do
                if storage.milestones[item.name] == nil and produced[item.name] and produced[item.name] >= settings.global["MileStoneSaves_threshold"].value then
                    storage.milestones[item.name] = true
                    ServerSave(item.name)
                end
            end
        end
    end
end

local function OnResearchFinished(event)
    if settings.global["MileStoneSaves_research"].value and storage.milestones[event.research.name] == nil then
        storage.milestones[event.research.name] = true
        ServerSave(event.research.name)
    end
end

local function OnRocketLaunched(event)
    if storage.milestones["RocketLaunched"] == nil then
        storage.milestones["RocketLaunched"] = true
        ServerSave("RocketLaunched")
    end
end

local function OnInit()
    storage.milestones = storage.milestones or {}
end

local function OnStartup()
    OnInit()
end

local function OnLoad()
    storage.milestones = storage.milestones or {}
end

script.on_init(OnStartup)
script.on_load(OnLoad)

script.on_nth_tick(settings.startup["MileStoneSaves_periodic"].value, IntervalServerSave)
script.on_nth_tick(settings.startup["MileStoneSaves_productivity"].value, ProductionChecks)

script.on_event(defines.events.on_research_finished, OnResearchFinished)
script.on_event(defines.events.on_rocket_launched, OnRocketLaunched)
