
local tint = settings.startup["color-mask"].value

local gui_tables = {
	["__core__/graphics/gui-new.png"] = {
		{0, 0, 152, 16}, {336, 0, 344, 3}, {0, 17, 33, 33}, {0, 40, 122, 47},
		{192, 8, 215, 15}, {248, 0, 264, 16}, {282, 0, 349, 16}, {208, 17, 224, 33},
		{248, 17, 298, 33}, {386, 17, 402, 33}, {369, 0, 419, 16}, {472, 0, 448, 16},
		{464, 17, 480, 33}, {489, 0, 568, 47}, {175, 40, 224, 47}, {247, 40, 265, 47},
		{296, 39, 304, 47}, {296, 48, 312, 64}, {415, 48, 431, 64}, {224, 48, 247, 67},
		{64, 48, 143, 71}, {0, 48, 19, 69}, {0, 72, 16, 88}, {17, 72, 29, 93},
		{56, 72, 72, 87}, {90, 72, 138, 88}, {313, 71, 329, 87}, {344, 78, 377, 102},
		{379, 78, 402, 85}, {395, 86, 411, 102}, {446, 78, 479, 102}, {0, 96, 63, 127},
		{128, 96, 155, 123}, {345, 103, 410, 135}, {448, 103, 464,119}, {0, 132, 55, 159},
		{256, 136, 288, 168}, {272, 169, 288, 185}, {496, 136, 496, 431}, {0, 160, 23, 183},
		{96, 160, 119, 183}, {0, 189, 47, 223}, {168, 200, 201, 216}, {296, 204, 414, 220},
		{0, 232, 191, 295}, {240, 424, 264, 456}, {240, 458, 264, 490}, {240, 492, 264, 524},
		{0, 424, 79, 503}, {313, 619, 329, 635}, {364, 619, 380, 635}, {413, 619, 465, 635},
		{448, 602, 466, 618}, {313, 639, 482, 655}, {0, 736, 79, 815}, {312, 744, 345, 760},
		{480, 816, 559, 895}, {0, 930, 79, 1009}, {0, 1040, 159, 1199}, {0, 1201, 16, 1217},
		{0, 1219, 596, 1219}, {0, 600, 31, 631}, {0, 584, 15, 600}, {48, 584, 96, 600},
		{138, 200, 141, 215}, {437, 17, 464, 48}
	},
	
	["__core__/graphics/gui.png"] = {
		{0, 0, 6, 6}, {8, 0, 14, 6}, {16, 0, 22, 6}, {8, 8, 14, 14},
		{16, 8, 22, 14}, {0, 16, 6, 22}, {0, 40, 6, 46}, {16, 24, 22, 30},
		{26, 17, 41, 32}, {43, 17, 58, 32}, {43, 34, 52, 43}, {65, 34, 74, 43},
		{77, 17, 92, 32}, {103, 0, 109, 14}, {24, 0, 93, 14}, {111, 0, 220, 35},
		{111, 114, 146, 179}
	},
	
	["__flib__/graphics/slots.png"] = {
		{0, 0, 79, 79}, {0, 80, 79, 159}, {240, 0, 319, 79}, {240, 80, 319, 159}
	},
	
	["__flib__/graphics/technology-slots.png"] = {
		{0, 800, 1151, 999}
	},
	
	["__helmod__/graphics/frame.png"] = {
		{0, 0, 31, 798}
	},
	
	["__helmod__/graphics/gui.png"] = {
		{8, 0, 30, 14}, {0, 32, 14, 38}, {0, 80, 54, 86}
	}
}

local function has_position(tbl)
    return tbl.position ~= nil
end


local function find_position_paths(tbl, current_path, result)
    result = result or {}
    current_path = current_path or ""
    for key, value in pairs(tbl) do
        local new_path = current_path == "" and key or current_path .. "." .. key
        if type(value) == "table" then
            if has_position(value) then
                table.insert(result, new_path)
            end
            find_position_paths(value, new_path, result)
        end
    end
    
    return result
end


function get_position_paths(style_data)
    return find_position_paths(style_data)
end

function tint_sprite(sprite)
	sprite.type = "composition"
	sprite.tint = tint
end


function sprite_check_core(style_table, file_name, pixel_areas)
	if style_table and style_table.position and (not style_table.filename or style_table.filename == file_name) then
		local sprite_x = style_table["position"][1] or style_table["position"]["x"]
		local sprite_y = style_table["position"][2] or style_table["position"]["y"]
				
		for _, pixel_area in pairs(pixel_areas) do
			if pixel_area[1] <= sprite_x and sprite_x <= pixel_area[3] and
			   pixel_area[2] <= sprite_y and sprite_y <= pixel_area[4] then
				tint_sprite(style_table)

				if settings.startup["color-mask"].value["a"] < 1 then
					style_table.background_blur = true
						
				end
			end
		end
	end
end

function sprite_check_other(style_table, file_name, pixel_areas)
	if style_table and style_table.position and style_table.filename == file_name then
		local sprite_x = style_table["position"][1] or style_table["position"]["x"]
		local sprite_y = style_table["position"][2] or style_table["position"]["y"]
				
		for _, pixel_area in pairs(pixel_areas) do
			if pixel_area[1] <= sprite_x and sprite_x <= pixel_area[3] and
			   pixel_area[2] <= sprite_y and sprite_y <= pixel_area[4] then
				tint_sprite(style_table)

				if settings.startup["color-mask"].value["a"] < 1 then
					style_table.background_blur = true
						
				end
			end
		end
	end
end

function change_sprite(path)
	local path_default = path:match("^([^%.]+)")
	local path_sprite = path:match("^[^%.]+%.(.*)")	
	local keys = {}
	for key in path_sprite:gmatch("[^%.]+") do
		table.insert(keys, key)
	end	
	local style_table = data.raw["gui-style"].default[path_default]
	if style_table then
		for i, key in ipairs(keys) do
			style_table = style_table[key]
		end
		--==for gui-new.png==--
		for file_name, pixel_areas in pairs(gui_tables) do
			if file_name == "__core__/graphics/gui-new.png" then
				sprite_check_core(style_table, file_name, pixel_areas)
			else
				sprite_check_other(style_table, file_name, pixel_areas)
			end
		end		
	end	

end

local function tint_sprite(sprite)
	if sprite then
		sprite.tint = tint
	end
end



if settings.startup["enable-color-mask"].value then
	data.raw["gui-style"].default["entity_frame_filler"].graphical_set = {}
	data.raw["gui-style"].default["shallow_slots_scroll_pane"].graphical_set = {}
	data.raw["gui-style"].default["factoriopedia_icon_grid_scroll_pane"].graphical_set = {}
	data.raw["gui-style"].default["tool_bar_empty_slot"].graphical_set = {}
	data.raw["gui-style"].default["empty_bonus_card_frame"].graphical_set = {}
	data.raw["gui-style"].default["technology_list_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["statistics_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["research_queue_table"].background_graphical_set = {}
	data.raw["gui-style"].default["list_box_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["pins_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["train_schedule_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["train_interrupts_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["mods_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["deep_slots_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["achievement_slots_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["logistic_gui_items_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["filter_tabbed_pane"].tab_container.background_graphical_set = {}
	data.raw["gui-style"].default["editor_mode_selection_table"].background_graphical_set = {}
	data.raw["gui-style"].default["slot_button_deep_frame"].background_graphical_set = {}
	data.raw["gui-style"].default["shortcut_bar_inner_panel"].background_graphical_set = {}
	data.raw["gui-style"].default["shortcut_bar_selection_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["side_menu_frame"].background_graphical_set = {}
	data.raw["gui-style"].default["trains_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["stations_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["trains_scroll_pane_with_side_margin"].background_graphical_set = {}
	data.raw["gui-style"].default["decider_combinator_conditions_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["decider_combinator_outputs_scroll_pane"].background_graphical_set = {}
	data.raw["gui-style"].default["scroll_pane"].graphical_set = {}
	
	--For toolbars-mod
	if data.raw["gui-style"].default["toolbar_content_sections_section_content_table"] then
		data.raw["gui-style"].default["toolbar_content_sections_section_content_table"].background_graphical_set = {}
	end
	
	--For cybersyn
	if data.raw["gui-style"].default["ltnm_small_slot_table_frame_light"] then
		data.raw["gui-style"].default["ltnm_small_slot_table_frame_light"].background_graphical_set = {}
	end
	if data.raw["gui-style"].default["ltnm_small_slot_table_frame_dark"] then
		data.raw["gui-style"].default["ltnm_small_slot_table_frame_dark"].background_graphical_set = {}
	end
	if data.raw["gui-style"].default["ltnm_slot_table_scroll_pane"] then
		data.raw["gui-style"].default["ltnm_slot_table_scroll_pane"].background_graphical_set = {}
	end
	
	--For cybersyn-combinator
	if data.raw["gui-style"].default["cybersyn-combinator_network-list_scroll-pane"] then
		data.raw["gui-style"].default["cybersyn-combinator_network-list_scroll-pane"].background_graphical_set = {}
	end
	
	local style_data = data.raw["gui-style"].default
	local styles = get_position_paths(style_data)

	for _, path in ipairs(styles) do
		change_sprite(path)
	end
	
	--Minor edits
	tint_sprite(data.raw["gui-style"].default["technology_gui_outer_frame"].header_background)

	tint_sprite(data.raw["gui-style"].default["map_preview_scroll_pane"].graphical_set.base.right)
	tint_sprite(data.raw["gui-style"].default["map_preview_scroll_pane"].graphical_set.base.right_bottom)
	tint_sprite(data.raw["gui-style"].default["map_preview_scroll_pane"].graphical_set.base.bottom)
	tint_sprite(data.raw["gui-style"].default["map_preview_scroll_pane"].graphical_set.base.left_bottom)
	tint_sprite(data.raw["gui-style"].default["map_preview_scroll_pane"].graphical_set.base.left)
	tint_sprite(data.raw["gui-style"].default["map_preview_scroll_pane"].graphical_set.base.center)
	tint_sprite(data.raw["gui-style"].default["tab_shallow_scroll_pane"].graphical_set.base.center)
	tint_sprite(data.raw["gui-style"].default["tab_shallow_scroll_pane"].graphical_set.base.top)
	
	--For utility-sprites
	tint_sprite(data.raw["utility-sprites"].default["slot"])
	tint_sprite(data.raw["utility-sprites"].default["equipment_slot"])
		
	--For FNEI
	if data.raw["gui-style"].default["fnei_back_button_style"] then
		tint_sprite(data.raw["gui-style"].default["fnei_back_button_style"].clicked_graphical_set)
		tint_sprite(data.raw["gui-style"].default["fnei_back_button_style"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_default_empty-tab"] then
		tint_sprite(data.raw["gui-style"].default["fnei_default_empty-tab"].clicked_graphical_set)
		tint_sprite(data.raw["gui-style"].default["fnei_default_empty-tab"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_default_selected-tab"] then
		tint_sprite(data.raw["gui-style"].default["fnei_default_selected-tab"].clicked_graphical_set)
		tint_sprite(data.raw["gui-style"].default["fnei_default_selected-tab"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_exit_button_style"] then
		tint_sprite(data.raw["gui-style"].default["fnei_exit_button_style"].clicked_graphical_set)
		tint_sprite(data.raw["gui-style"].default["fnei_exit_button_style"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_hotbar_down_arrow"] then
		tint_sprite(data.raw["gui-style"].default["fnei_hotbar_down_arrow"].clicked_graphical_set)
		tint_sprite(data.raw["gui-style"].default["fnei_hotbar_down_arrow"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_hotbar_up_arrow"] then
		tint_sprite(data.raw["gui-style"].default["fnei_hotbar_up_arrow"].clicked_graphical_set)
		tint_sprite(data.raw["gui-style"].default["fnei_hotbar_up_arrow"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_left_arrow_button_style"] then
		tint_sprite(data.raw["gui-style"].default["fnei_left_arrow_button_style"].clicked_graphical_set)
		tint_sprite(data.raw["gui-style"].default["fnei_left_arrow_button_style"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_recipe_favorive_button"] then
		tint_sprite(data.raw["gui-style"].default["fnei_recipe_favorive_button"].clicked_graphical_set)
		tint_sprite(data.raw["gui-style"].default["fnei_recipe_favorive_button"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_recipe_selected_favorive_button"] then
		tint_sprite(data.raw["gui-style"].default["fnei_recipe_selected_favorive_button"].clicked_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_right_arrow_button_style"] then
		tint_sprite(data.raw["gui-style"].default["fnei_right_arrow_button_style"].clicked_graphical_set)
		tint_sprite(data.raw["gui-style"].default["fnei_right_arrow_button_style"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_settings_button_style"] then
		tint_sprite(data.raw["gui-style"].default["fnei_settings_button_style"].clicked_graphical_set)
		tint_sprite(data.raw["gui-style"].default["fnei_settings_button_style"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_settings_down_arrow"] then
		tint_sprite(data.raw["gui-style"].default["fnei_settings_down_arrow"].default_graphical_set)
	end
	if data.raw["gui-style"].default["fnei_settings_up_arrow"] then
		tint_sprite(data.raw["gui-style"].default["fnei_settings_up_arrow"].default_graphical_set)
	end

	--For GUI_Unifier
	if data.raw["gui-style"].default["gui_Unifier_gui_01"] then
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_01"].default_graphical_set)
	end
	if data.raw["gui-style"].default["gui_Unifier_gui_02"] then
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_02"].default_graphical_set)
	end
	if data.raw["gui-style"].default["gui_Unifier_gui_03"] then
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_03"].default_graphical_set)
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_03"].hovered_graphical_set)
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_03"].selected_graphical_set)
	end
	if data.raw["gui-style"].default["gui_Unifier_gui_04"] then
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_04"].default_graphical_set)
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_04"].hovered_graphical_set)
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_04"].selected_graphical_set)
	end
	if data.raw["gui-style"].default["gui_Unifier_gui_05"] then
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_05"].default_graphical_set)
	end
	if data.raw["gui-style"].default["gui_Unifier_gui_06"] then
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_06"].default_graphical_set)
	end
	if data.raw["gui-style"].default["gui_Unifier_gui_07"] then
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_07"].default_graphical_set)
	end
	if data.raw["gui-style"].default["gui_Unifier_gui_08"] then
		tint_sprite(data.raw["gui-style"].default["gui_Unifier_gui_08"].default_graphical_set)
	end
	if data.raw["gui-style"].default["todo_button_default_snouz"] then
		tint_sprite(data.raw["gui-style"].default["todo_button_default_snouz"].default_graphical_set)
		tint_sprite(data.raw["gui-style"].default["todo_button_default_snouz"].disabled_graphical_set)
	end
	
	--For toolbars-mod
	if data.raw["gui-style"].default["toolbar_content_sections_section_content_table_row_slot_item_overlay_count"] then
		data.raw["gui-style"].default["toolbar_content_sections_section_content_table_row_slot_item_overlay_count"].default_graphical_set.base.background_blur = false
	end
	if data.raw["gui-style"].default["toolbar_content_sections_section_content_table_row_slot_item_overlay_dim"] then
		data.raw["gui-style"].default["toolbar_content_sections_section_content_table_row_slot_item_overlay_dim"].default_graphical_set.base.background_blur = false
	end
	if data.raw["gui-style"].default["toolbar_content_sections_section_content_table_row_slot_item_overlay_dim_some"] then
		data.raw["gui-style"].default["toolbar_content_sections_section_content_table_row_slot_item_overlay_dim_some"].default_graphical_set.base.background_blur = false
	end
	if data.raw["gui-style"].default["toolbar_content_sections_section_content_table_row_slot_item_overlay_dim_none"] then
		data.raw["gui-style"].default["toolbar_content_sections_section_content_table_row_slot_item_overlay_dim_none"].default_graphical_set.base.background_blur = false
	end
end
